@@ -1,6 +1,6 @@
 /******************************************************************************
-  This source file is part of the Avogadro project.
-  This source code is released under the 3-Clause BSD License, (see "LICENSE").
+ This source file is part of the Avogadro project.
+ This source code is released under the 3-Clause BSD License, (see "LICENSE").
 ******************************************************************************/
 
 #include "mainwindow.h"
@@ -112,7 +112,21 @@ public:
   {
     this->XMLStream = nullptr;
   }
-  ~XMLEventObserver() { delete this->XMLStream; }
+  ~XMLEventObserver()
+  {
+    if (this->XMLStream) {
+      this->XMLStream->writeEndElement(); // close properly
+      this->XMLStream->writeEndDocument();
+      delete this->XMLStream;
+      this->XMLStream = nullptr;
+    }
+
+    // Flush any remaining content to the stream
+    if (this->Stream && !this->XMLString.isEmpty()) {
+      *this->Stream << this->XMLString;
+      this->Stream->flush(); // Ensure it's written
+    }
+  }
 
 protected:
   virtual void setStream(QTextStream* stream) override
@@ -123,8 +137,10 @@ protected:
       delete this->XMLStream;
       this->XMLStream = nullptr;
     }
-    if (this->Stream)
+    if (this->Stream) {
       *this->Stream << this->XMLString;
+      this->Stream->flush(); // make sure to flush
+    }
 
     this->XMLString = QString();
     pqEventObserver::setStream(stream);
@@ -197,6 +213,8 @@ protected:
     widget = this->XMLStream->attributes().value("widget").toString();
     command = this->XMLStream->attributes().value("command").toString();
     arguments = this->XMLStream->attributes().value("arguments").toString();
+    eventType =
+      this->XMLStream->attributes().value("eventType").toString().toInt();
     return SUCCESS;
   }
 };
@@ -307,6 +325,12 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
 #endif
   setupInterface();
 
+#ifdef QTTESTING
+  m_testUtility = new pqTestUtility(this);
+  m_testUtility->addEventObserver("xml", new XMLEventObserver(this));
+  m_testUtility->addEventSource("xml", new XMLEventSource(this));
+#endif
+
   // Build up the standard menus, incorporate dynamic menus.
 #ifdef Q_OS_WIN
   qDebug() << " building menu ";
@@ -536,6 +560,37 @@ void MainWindow::setupInterface()
           &MainWindow::viewActivated);
 }
 
+void MainWindow::closeActiveMolecule()
+{
+  if (!saveFileIfNeeded()) {
+    return;
+  }
+
+  Molecule* currentMol = m_molecule;
+  const QList<Molecule*> molecules = m_moleculeModel->molecules();
+  const int idx = molecules.indexOf(currentMol);
+
+  if (idx < 0 || molecules.isEmpty()) {
+    return;
+  }
+
+  if (idx == 0) {
+    // if the current molecule is the first one in the list, and it is not the
+    // only one we make the next molecule the starting one
+    if (molecules.size() > 1) {
+      setMolecule(molecules[idx + 1]);
+    } else {
+      newMolecule();
+    }
+  }
+  // otherwise we just set the current molecule to the one before
+  else {
+    setMolecule(molecules[idx - 1]);
+  }
+
+  m_moleculeModel->removeItem(currentMol);
+}
+
 void MainWindow::closeEvent(QCloseEvent* e)
 {
   writeSettings();
@@ -565,6 +620,10 @@ void MainWindow::changeEvent(QEvent* e)
       if (tool != nullptr)
         tool->setIcon(darkMode);
     }
+
+    // change the theme of the molecule model
+    if (m_moleculeModel != nullptr)
+      m_moleculeModel->loadIcons(darkMode);
   }
 
   QMainWindow::changeEvent(e);
@@ -624,7 +683,8 @@ void setWidgetMolecule(T* glWidget, M* mol)
 {
   glWidget->setMolecule(mol);
   glWidget->updateScene();
-  glWidget->resetCamera();
+  if (mol->atomCount() == 0)
+    glWidget->resetCamera();
 }
 
 void setDefaultViews(MultiViewWidget* viewWidget)
@@ -1199,6 +1259,12 @@ void MainWindow::layerActivated(const QModelIndex& idx)
 
 void MainWindow::moleculeActivated(const QModelIndex& idx)
 {
+  QList<Molecule*> molecules = m_moleculeModel->molecules();
+  if (idx.isValid() && idx.row() == molecules.size()) {
+    newMolecule();
+    return;
+  }
+
   auto* obj = static_cast<QObject*>(idx.internalPointer());
   if (auto* mol = qobject_cast<Molecule*>(obj)) {
     if (idx.column() == 0)
@@ -1207,7 +1273,6 @@ void MainWindow::moleculeActivated(const QModelIndex& idx)
     // Deleting a molecule, we must also create a new one if it is the last.
     if (idx.column() == 1) {
       if (m_molecule == mol) {
-        QList<Molecule*> molecules = m_moleculeModel->molecules();
         int molIdx = molecules.indexOf(mol);
         if (molIdx > 0)
           setMolecule(molecules[molIdx - 1]);
@@ -1601,8 +1666,8 @@ bool MainWindow::saveFileAs(bool async)
 
   QFileDialog saveDialog(this, tr("Save chemical file"), dir, filter);
   saveDialog.setAcceptMode(QFileDialog::AcceptSave);
-  saveDialog.exec();
-  if (saveDialog.selectedFiles().isEmpty()) // user cancel
+  bool ok = saveDialog.exec();
+  if (!ok || saveDialog.selectedFiles().isEmpty()) // user cancel
     return false;
 
   QString fileName = saveDialog.selectedFiles().first();
@@ -2022,19 +2087,14 @@ void MainWindow::buildMenu()
   QStringList testingPath;
   testingPath << tr("&Testing");
   QAction* actionRecord = new QAction(this);
-  actionRecord->setText(tr("Record test…"));
+  actionRecord->setText(tr("&Record Test…"));
   m_menuBuilder->addAction(testingPath, actionRecord, 10);
   QAction* actionPlay = new QAction(this);
-  actionPlay->setText(tr("Play test…"));
+  actionPlay->setText(tr("&Play Test…"));
   m_menuBuilder->addAction(testingPath, actionPlay, 5);
 
   connect(actionRecord, SIGNAL(triggered()), SLOT(record()));
   connect(actionPlay, SIGNAL(triggered()), SLOT(play()));
-
-  m_testUtility = new pqTestUtility(this);
-  m_testUtility->addEventObserver("xml", new XMLEventObserver(this));
-  m_testUtility->addEventSource("xml", new XMLEventSource(this));
-
   m_testExit = true;
 #endif
 
@@ -2063,12 +2123,13 @@ void MainWindow::buildMenu()
   m_fileToolBar->addAction(action);
   connect(action, &QAction::triggered, this, &MainWindow::importFile);
 
+  // close current molecule
   action = new QAction(tr("&Close"), this);
   action->setShortcut(QKeySequence::Close);
   action->setIcon(QIcon::fromTheme("document-close"));
   m_menuBuilder->addAction(path, action, 981);
   m_fileToolBar->addAction(action);
-  connect(action, &QAction::triggered, this, &QWidget::close);
+  connect(action, &QAction::triggered, this, &MainWindow::closeActiveMolecule);
 
   // Separator (after open recent)
   action = new QAction("", this);
@@ -2546,21 +2607,50 @@ void MainWindow::finishUpdateRequest(QNetworkReply* reply)
   qDebug() << " update comparing " << lastVersion << " to " << latestRelease;
 #endif
   QStringList releaseComponents = latestRelease.split('.');
-  QStringList currentComponents = lastVersion.split('.');
-  if (releaseComponents.size() != 3 || currentComponents.size() != 3)
+  QStringList lastComponents = lastVersion.split('.');
+  QStringList currentComponents = QString(AvogadroApp_VERSION).split('.');
+
+  if (releaseComponents.size() != 3 || lastComponents.size() != 3 ||
+      currentComponents.size() != 3)
     // something is very wrong
     return;
 
-  if (currentComponents[0] > releaseComponents[0] ||
-      (currentComponents[0] == releaseComponents[0] &&
-       currentComponents[1] > releaseComponents[1]))
-    // no update needed
-    return;
+  int lastMajor = lastComponents[0].toInt();
+  int lastMinor = lastComponents[1].toInt();
+  int lastPatch = lastComponents[2].toInt();
+  int currentMajor = currentComponents[0].toInt();
+  int currentMinor = currentComponents[1].toInt();
+  int currentPatch = currentComponents[2].toInt();
+  int releaseMajor = releaseComponents[0].toInt();
+  int releaseMinor = releaseComponents[1].toInt();
+  int releasePatch = releaseComponents[2].toInt();
 
-  if (currentComponents[0] == releaseComponents[0] &&
-      currentComponents[1] == releaseComponents[1] &&
-      currentComponents[2] >= releaseComponents[2]) {
+  // okay, decide if we're using the version from settings
+  // or the compiled version to determine if an update is available
+  // e.g., if currentComponents is newer than last
+  // we should save the current version in settings
+  // and use that to see if there's an update
+
+  if (currentMajor > lastMajor ||
+      (currentMajor == lastMajor && currentMinor > lastMinor) ||
+      (currentMajor == lastMajor && currentMinor == lastMinor &&
+       currentPatch > lastPatch)) {
+    settings.setValue("currentVersion", AvogadroApp_VERSION);
+    settings.sync();
+  }
+
+  // they're presumably the same now, so let's see if the current
+  // version is newer than the latest release
+
+  if (currentMajor > releaseMajor ||
+      (currentMajor == releaseMajor && currentMinor > releaseMinor) ||
+      (currentMajor == releaseMajor && currentMinor == releaseMinor &&
+       currentComponents[2] >= releaseComponents[2])) {
     // this will work for like "0-36-whatever" > "0" but not "1"
+#ifdef NDEBUG
+    qDebug() << "current version is newer than latest release";
+#endif
+    // no update needed
     return;
   }
 
